#!/usr/local/perl5/bin/perl -w

# MIT local addition:
# This script handles the authentication options page.  It allows
# the user to test the available REMOTE_USER authentication options
# (i.e. kerberos and certificate), as well as to set the preference
# for which authentication method to use by default.

use CGI ();
use CGI::Cookie ();
use CGI::Fast ();
use HTML::Template ();
use WebKDC::Config ();

our $DEBUG = 0;

# The name of the cookie holding REMOTE_USER configuration information.
our $REMUSER_COOKIE = 'weblogin_remuser';

# The lifetime of the REMOTE_USER configuration cookie.
our $REMUSER_LIFETIME = '+365d';

# Open the HTML template.
my $template = HTML::Template->new(filename => 'auth-options.tmpl',
                                   path => $WebKDC::Config::TEMPLATE_PATH);

# The main loop.  If we're not running under FastCGI, CGI::Fast will detect
# that and only run us through the loop once.  Otherwise, we live in this
# processing loop until the FastCGI socket closes.
while (my $q = CGI::Fast->new) {

    my $remuser_tried = $q->param ('remuser_tried') || '';
    my $remuser_new_pref = $q->param('remuser_pref') || '';
    my $save_remuser_pref = $q->param('save_remuser_pref') || '';
    my $remuser_current_pref = '';
    my $cookie;

    # User said to save preferences.
    if ($save_remuser_pref) {
        my $value = '';
        print STDERR "remuser_new_pref is " . $remuser_new_pref . "\n"
            if $DEBUG;
        if ($remuser_new_pref ne 'none') {
            $value = $remuser_new_pref;
        }
        $cookie = $q->cookie(-name => $REMUSER_COOKIE, -value => $value,
                             -secure => 1,
                             -expires => $REMUSER_LIFETIME);
        # Show the new value as the current setting.
        $remuser_current_pref = $remuser_new_pref;
        $template->param(preference_saved => 1);
    } else {
        # Not saving, get the current setting.
        $remuser_current_pref = $q->cookie ($REMUSER_COOKIE) || '';
    }
    print STDERR "current pref is " . $remuser_current_pref . "\n"
        if $DEBUG;

    if ($remuser_current_pref eq 'kerberos') {
        $template->param(remuser_kerberos => 1);
    } elsif ($remuser_current_pref eq 'certificate') {
        $template->param(remuser_certificate => 1);
    } else {
        $template->param(remuser_none => 1);
    }
    print STDERR "remuser_tried is " . $remuser_tried . "\n"
        if $DEBUG;
    if ($remuser_tried) {
        $template->param(($remuser_tried . "_tried") => 1);
        $template->param(remuser_name => $ENV{REMOTE_USER});
    }

    # Pass the script name.
    print STDERR "script_name is " .  $q->script_name . "\n" if $DEBUG;
    $template->param(script_name => $q->script_name);
    while (($type, $url) = each %WebKDC::Config::REMOTE_USER_TESTS) {
        my $param_name = "test_" . ${type} . "_url";
        $template->param($param_name => $url);
    }

    # Now, print out the page header with the appropriate cookie.
    if ($cookie) {
        print $q->header (-type => 'text/html', -Pragma => 'no-cache',
                          -Cache_Control => 'no-cache, no-store',
                          -cookie => $cookie);
    } else {
        print $q->header (-type => 'text/html', -Pragma => 'no-cache',
                          -Cache_Control => 'no-cache, no-store');
    }
    print $template->output;

# Done on each pass through the FastCGI loop.  Clear out template parameters
# for all of the pages for the next run and restart the script if its
# modification time has changed.
} continue {
    $template->clear_params;
    exit if -M $ENV{SCRIPT_FILENAME} < 0;
}
